#! /usr/bin/env python
# -*- Mode: Python -*-
# -*- coding: ascii -*-

"""
This is a LightWave Generic plug-in that will 
load a surface from a file and apply it to a sequence of object files
based on the currently selected object 
Except that the process of loading a surface and assigning it causes
layout to crash.  So instead it copies the surface from another object in the scene.
"""

import sys
import lwsdk
import os
import re


__author__     = "Clinton Reese"
__date__       = "November 8 2013"
__copyright__  = "Clinton's 3D Creations"
__version__    = "1.2"
__maintainer__ = "Clinton Reese"
__email__      = "reese@clintons3d.com"
__status__     = "Unstable"
__lwver__      = "11.5"

#CRASHES IF SEQUENCE IS ACTIVE

class sequence_surface_loader(lwsdk.IGeneric):
    def __init__(self, context):
        super(sequence_surface_loader, self).__init__()

        #ui variables
        self._SequenceOff = 0 # 0 is NO - object sequence is active
        self._firstFrame = -1
        self._lastFrame = -1
        self._sourceItemID = None
        self._sequenceItemID = None
        
        self._filepath = ""
        self._numDigits = 1

        self.surface_funcs = lwsdk.LWSurfaceFuncs()
        #match one or more digits , "." and "lwo" at the end
        #try with any extension
        #self._reCompile = re.compile(r'\d+\.lwo$') 
        self._reCompile = re.compile(r'\d+\.\D{3}$')
        self._fileExt = "lwo"
        
        #item name with (#) appended will crash
        self._reCrashComing = re.compile(r'\(\d+\)')
        
        self._coreConsole = lwsdk.LWPCoreConsole()
        #self._monitor_funcs = lwsdk.DynaMonitorFuncs()
        self._UserCancel = False
        self._monitor = None
        
    def copySurfaceNames(self):
        numSourceSurfacesToCopy = lwsdk.LWSurfaceFuncs().slibCount(self._sourceSurfaceLib)
        numSequenceSurfacesAvailable = lwsdk.LWSurfaceFuncs().slibCount(self._sequenceSurfaceLib)
        if numSourceSurfacesToCopy >= numSequenceSurfacesAvailable:
            copyCount = numSequenceSurfacesAvailable
        else:
            copyCount = numSourceSurfacesToCopy
        
        for si in range(0, copyCount):
            sourceSurf = lwsdk.LWSurfaceFuncs().slibByIndex(self._sourceSurfaceLib, si)
            surfname = lwsdk.LWSurfaceFuncs().name(sourceSurf)
            sequenceSurf = lwsdk.LWSurfaceFuncs().slibByIndex(self._sequenceSurfaceLib, si)
            lwsdk.LWSurfaceFuncs().rename(sequenceSurf, surfname)


    def processFiles(self, frame, ga):
        frameS_digits = '%i' % self._numDigits
        #frameS_template = '%0' + frameS_digits + 'i.lwo'
        frameS_template = '%0' + frameS_digits + 'i.' + self._fileExt
        frameS = frameS_template % frame
        checkfile = self._reCompile.sub(frameS, self._filepath)
        
        if self._monitor is not None:
            if lwsdk.LWLMonFuncs().step( self._monitor, 1, checkfile ):
                lwsdk.LWLMonFuncs().step( self._monitor, 0, "User Aborted!" )
                self._UserCancel = True
                return

        ga.evaluate(ga.data, "LoadObject %s" % checkfile)
        
        #looks like item is selected when loaded
        selected_items = lwsdk.LWInterfaceInfo().selected_items()
        item_ID = selected_items[0]
        sequenceItemName = lwsdk.LWItemInfo().name(item_ID)
        self._sequenceSurfaceLib = lwsdk.LWSurfaceFuncs().objectLib(sequenceItemName)
        lwsdk.LWSurfaceFuncs().copyLib(self._sequenceSurfaceLib, self._sourceSurfaceLib)
        
        self.copySurfaceNames()
        
        ga.evaluate(ga.data, "SaveObject %s" % checkfile)
        
        clearItemsAndDescendantsWithoutAsking = 3
        ga.evaluate(ga.data, "ClearSelected %i" % clearItemsAndDescendantsWithoutAsking)
            
        
    def FindFrameRange(self, framestring):
        selectedframe = int(framestring)

        self._firstFrame = selectedframe
        self._lastFrame = selectedframe
        
        prevframe = self._firstFrame
        
        checkfile = self._filepath
        while os.path.exists(checkfile):
            self._firstFrame = prevframe
            prevframe = prevframe - 1
            frameS_digits = '%i' % self._numDigits
            #frameS_template = '%0' + frameS_digits + 'i.lwo'
            frameS_template = '%0' + frameS_digits + 'i.' + self._fileExt
            frameS = frameS_template % prevframe
            checkfile = self._reCompile.sub(frameS, self._filepath)
            
        nextframe = self._lastFrame
        checkfile = self._filepath
        while os.path.exists(checkfile):
            self._lastFrame = nextframe
            nextframe = nextframe + 1
            frameS_digits = '%i' % self._numDigits
            #frameS_template = '%0' + frameS_digits + 'i.lwo'
            frameS_template = '%0' + frameS_digits + 'i.' + self._fileExt
            frameS = frameS_template % nextframe
            checkfile = self._reCompile.sub(frameS, self._filepath)

        return True

    # LWGeneric -------------------------------------------
    def process(self, ga):
        # panels
        # /LightWave11.0/sdk/lwsdk11.0/html/globals/panel.html
        # /LightWave11.0/sdk/lwsdk11.0/include/lwpanel.h
        ui = lwsdk.LWPanels()
        panel = ui.create('Sequenced Object Surface Loader')

        controlWidth = 64

        #c0 = panel.hchoice_ctl('Is the Object Replacement set to None?',("No","Yes"))
        c1 = panel.item_ctl('copy surface from', lwsdk.LWI_OBJECT)
        c2 = panel.item_ctl('sequenced object', lwsdk.LWI_OBJECT)
        c3 = panel.int_ctl('First frame')
        c4 = panel.int_ctl('Last  frame')

        #c0.set_int(0)
        c3.set_int(self._firstFrame)
        c4.set_int(self._lastFrame)

        if panel.open(lwsdk.PANF_BLOCKING | lwsdk.PANF_CANCEL) == 0:
            ui.destroy(panel)
            return lwsdk.AFUNC_OK
            
        #self._SequenceOff = c0.get_int()
        #if self._SequenceOff == 0:
        #    print >>sys.stderr,'Set the Object Replacement to None and try again. "p" for properties and remove the Object Sequence'
        #    print >>sys.stderr,'If Object Replacement is not set to None, Layout will crash!'
        #    return lwsdk.AFUNC_OK
            
        self._sourceItemID = c1.get_addr()
        self._sequenceItemID = c2.get_addr()
        self._firstFrame =    c3.get_int()
        self._lastFrame =     c4.get_int()
        
        self._coreConsole.show()
        self._coreConsole.clear()
        
        if self._firstFrame > self._lastFrame:
            print >>sys.stderr,'Invalid frame range.\nFirst frame is greater than the last frame.'
            return lwsdk.AFUNC_OK
        
        if self._sourceItemID is None:
            print >>sys.stderr,'Select an item to copy the surface from.'
            return lwsdk.AFUNC_OK
            
        if self._sequenceItemID is None:
            print >>sys.stderr,'Select a sequence item to copy the surface to.'
            return lwsdk.AFUNC_OK
            
        sourceItemName = lwsdk.LWItemInfo().name(self._sourceItemID)
        print 'source item: ' + sourceItemName
        
        sequenceItemName = lwsdk.LWItemInfo().name(self._sequenceItemID)
        print 'sequence item: ' + sequenceItemName
        
        if sequenceItemName == sourceItemName:
            print >>sys.stderr,'Source and destination items cannot be the same.'
            return lwsdk.AFUNC_OK
        
        #prevent crash by rename
        #look for space + '('
        myReCompile = re.compile(r'\s\(\d')
        needChangeName = myReCompile.search(sequenceItemName)
        if needChangeName is not None:
            seqNameArray = sequenceItemName.split(' ')
            sequenceItemName = seqNameArray[0]
            print 'new sequence item name: ' + sequenceItemName
        
        boo = self._reCrashComing.search(sequenceItemName)
        if boo is not None:
            print >>sys.stderr,'Sequence item name problem. Save and clear the scene file with the sequence set to None before running this script.'
            return lwsdk.AFUNC_OK
                
        self._sourceSurfaceLib = lwsdk.LWSurfaceFuncs().objectLib(sourceItemName)
        self._sequenceSurfaceLib = lwsdk.LWSurfaceFuncs().objectLib(sequenceItemName)
        
        #this is where the object sequence crash occurs - seq name with '(#)' append is cause
        lwsdk.LWSurfaceFuncs().copyLib(self._sequenceSurfaceLib, self._sourceSurfaceLib)
        
        self.copySurfaceNames()

        self._filepath = lwsdk.LWObjectInfo().filename(self._sequenceItemID)
        print self._filepath
        ga.evaluate(ga.data, "SelectItem %s" % lwsdk.itemid_to_str(self._sequenceItemID))
        ga.evaluate(ga.data, "SaveObject %s" % self._filepath)

        boo = self._reCompile.search(self._filepath);
        if boo is None:
            print >>sys.stderr,'Mesh is not part of a sequence.'
            return lwsdk.AFUNC_OK
        boogrp = boo.group()
        print boogrp
        
        boogrparray = boogrp.split('.')
        numstr = boogrparray[0]
        print numstr
        
        self._fileExt = boogrparray[1]
        
        self._numDigits = len(numstr)
        #if min frame = max frame = -1 determine range automatically
        if self._firstFrame == self._lastFrame == -1:
            if not self.FindFrameRange(numstr):
                return lwsdk.AFUNC_OK
        
        print "first frame = %i" % self._firstFrame
        print "last frame = %i" % self._lastFrame
        
        numFrames = self._lastFrame - self._firstFrame
        
        self._monitor = lwsdk.LWLMonFuncs().create()
        if self._monitor is not None:
            lwsdk.LWLMonFuncs().setup( self._monitor, "Sequence Surface Copy", lwsdk.LMO_REVIEW | lwsdk.LMO_IMMUPD, None )
            lwsdk.LWLMonFuncs().init( self._monitor, numFrames, "Starting..." )

        
        #don't process the current frame of the sequence - causes a crash?
        numactual = int(numstr)
        for i in range(self._firstFrame, self._lastFrame + 1):
            if i != numactual:
                self.processFiles(i, ga)
                if self._UserCancel:
                    print "User cancelled"
                    if self._monitor is not None:
                        lwsdk.LWLMonFuncs().done( self._monitor )
                        lwsdk.LWLMonFuncs().destroy( self._monitor )
                    return lwsdk.AFUNC_OK

        print 'done processing'
        if self._monitor is not None:
            lwsdk.LWLMonFuncs().done( self._monitor )
            lwsdk.LWLMonFuncs().destroy( self._monitor )
        
        return lwsdk.AFUNC_OK

ServerTagInfo = [
                    ( "Python Object Sequence Surface Loader", lwsdk.SRVTAG_USERNAME | lwsdk.LANGID_USENGLISH ),
                    ( "Object Sequence Surface Loader", lwsdk.SRVTAG_BUTTONNAME | lwsdk.LANGID_USENGLISH ),
                    ( "Utilities/Python", lwsdk.SRVTAG_MENU | lwsdk.LANGID_USENGLISH )
                ]

ServerRecord = { lwsdk.GenericFactory("LW_PyLoadSurfaceToSequence", sequence_surface_loader) : ServerTagInfo }
